@@ -1,26 +1,23 @@
 use strict;
 use warnings;
 package WWW::Spotify;
-$WWW::Spotify::VERSION = '0.006';
-
+$WWW::Spotify::VERSION = '0.007';
 use Moo 2.002004;
 
 use Data::Dumper;
+use File::Basename;
+use HTTP::Headers;
+use IO::CaptureOutput qw( capture qxx qxy );
+use JSON::Path;
+use JSON::XS;
+use MIME::Base64;
+use Scalar::Util;
+use Try::Tiny qw( catch try );
+use Types::Standard qw( Bool InstanceOf Int Str );
 use URI;
 use URI::Escape;
 use WWW::Mechanize;
-use JSON::XS;
-use JSON::Path;
 use XML::Simple;
-use HTTP::Headers;
-use Scalar::Util;
-use File::Basename;
-use IO::CaptureOutput qw( capture qxx qxy );
-use MIME::Base64;
-use Types::Standard qw( Int Str );
-
-#use LWP::Authen::OAuth2;
-#use Digest::MD5::File qw( file_md5_hex url_md5_hex );
 
 has 'oauth_authorize_url' => (
     is      => 'rw',
@@ -84,7 +81,7 @@ has 'results' => (
 
 has 'debug' => (
     is      => 'rw',
-    isa     => Int,
+    isa     => Bool,
     default => 0
 );
 
@@ -153,6 +150,14 @@ has problem => (
     default => q{}
 );
 
+has ua => (
+    is      => 'ro',
+    isa     => InstanceOf ['WWW::Mechanize'],
+    handles => { _mech => 'clone' },
+    lazy    => 1,
+    default => sub { WWW::Mechanize->new( autocheck => 0 ) },
+);
+
 my %api_call_options = (
 
     '/v1/albums/{id}' => {
@@ -291,22 +296,6 @@ foreach my $key ( keys %api_call_options ) {
     $method_to_uri{ $api_call_options{$key}->{method} } = $key;
 }
 
-# print Dumper(\%method_to_uri);
-
-sub is_valid_json {
-    my ( $self, $json, $caller ) = @_;
-    $caller ||= 'undefined method';
-    eval { decode_json $json; };
-
-    if ($@) {
-        $self->last_error("invalid json passed into '$caller'");
-        return 0;
-    }
-    else {
-        return 1;
-    }
-}
-
 sub send_post_request {
     my $self       = shift;
     my $attributes = shift;
@@ -404,7 +393,7 @@ sub send_get_request {
 
     warn "$url\n" if $self->debug;
     local $ENV{PERL_LWP_SSL_VERIFY_HOSTNAME} = 0;
-    my $mech = WWW::Mechanize->new( autocheck => 0 );
+    my $mech = $self->_mech;
 
     if ( $attributes->{client_auth_required} ) {
 
@@ -478,7 +467,6 @@ sub get_oauth_authorize {
 
     my $grant_type = 'authorization_code';
     local $ENV{PERL_LWP_SSL_VERIFY_HOSTNAME} = 0;
-    my $mech = WWW::Mechanize->new( autocheck => 0 );
     my $client_and_secret
         = $self->oauth_client_id() . ':' . $self->oauth_client_secret();
     my $encoded = encode_base64($client_and_secret);
@@ -491,21 +479,15 @@ sub get_oauth_authorize {
     $parts[0] = 'response_type=code';
     $parts[1] = 'redirect_uri=' . $self->oauth_redirect_uri;
 
-    # $parts[2] = 'state=' ;
-
     my $params = join( '&', @parts );
     $url = $url . "?client_id=" . $self->oauth_client_id() . "&$params";
 
-    # print $url , "\n";
-    $mech->get($url);
-
-    # print Dumper($mech);
-    return $mech->content();
+    $self->ua->get($url);
 
+    return $self->ua->content;
 }
 
 sub get_client_credentials {
-
     my $self  = shift;
     my $scope = shift;
 
@@ -518,7 +500,7 @@ sub get_client_credentials {
 
     my $grant_type = 'client_credentials';
     local $ENV{PERL_LWP_SSL_VERIFY_HOSTNAME} = 0;
-    my $mech = WWW::Mechanize->new( autocheck => 0 );
+    my $mech = $self->_mech;
     my $client_and_secret
         = $self->oauth_client_id() . ':' . $self->oauth_client_secret();
     my $encoded = encode_base64($client_and_secret);
@@ -596,7 +578,6 @@ sub get_access_token {
     # need to authorize first??
 
     local $ENV{PERL_LWP_SSL_VERIFY_HOSTNAME} = 0;
-    my $mech = WWW::Mechanize->new( autocheck => 0 );
     my $client_and_secret
         = $self->oauth_client_id() . ':' . $self->oauth_client_secret();
 
@@ -616,13 +597,12 @@ sub get_access_token {
         $extra->{scope} = $scope;
     }
 
+    my $mech = $self->_mech;
     $mech->add_header( 'Authorization' => 'Basic ' . $encoded );
 
     $mech->post( $url, [$extra] );
 
-    # print Dumper($mech);
     print $mech->content(), "\n";
-
 }
 
 sub get {
@@ -989,7 +969,7 @@ WWW::Spotify - Spotify Web API Wrapper
 
 =head1 VERSION
 
-version 0.006
+version 0.007
 
 =head1 SYNOPSIS
 
@@ -1073,6 +1053,23 @@ Have access to a JSON viewer to help develop and debug. The Chrome JSON viewer i
 very good and provides the exact path of the item within the JSON in the lower left
 of the screen as you mouse over an element.
 
+=head1 CONSTRUCTOR ARGS
+
+=head2 ua
+
+You may provide your own L<WWW::Mechanize> object to the constructor.  You may
+want to set autocheck off.  To get extra debugging information, you can do
+something like this:
+
+    use LWP::ConsoleLogger::Easy qw( debug_ua );
+    use WWW::Mechanize;
+    use WWW::Spotify;
+
+    my $mech = WWW::Mechanize->new( autocheck => 0 );
+    debug_ua( $mech );
+
+    my $ua = WWW::Mechanize->new( ua => $ua );
+
 =head1 METHODS
 
 =head2 get
